'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

var classnames = function classnames(cls) {
  var classList = [];

  if (Array.isArray(cls)) {
    classList = cls.map(function (item) {
      return item;
    });
  }

  if (Object.prototype.toString.call(cls) === '[object Object]') {
    Object.keys(cls).forEach(function (item) {
      if (cls[item]) {
        classList.push(item);
      }
    });
  }

  return classList.join(' ');
};

var getTransformValue = function getTransformValue(key, value) {
  return key + '(' + value + ')';
};

var transform = function transform() {
  var values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var result = [];

  Object.keys(values).forEach(function (key) {
    var value = Array.isArray(values[key]) ? values[key].join(', ') : values[key];

    result.push(getTransformValue(key, value));
  });

  return result.join(' ');
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var EffectDropdown = function (_Component) {
  inherits(EffectDropdown, _Component);

  function EffectDropdown() {
    classCallCheck(this, EffectDropdown);

    var _this = possibleConstructorReturn(this, (EffectDropdown.__proto__ || Object.getPrototypeOf(EffectDropdown)).call(this));

    _this.state = {
      active: false
    };

    _this.$el = null;

    _this.toggle = _this.toggle.bind(_this);
    _this.hide = _this.hide.bind(_this);
    _this.handleBodyClick = _this.handleBodyClick.bind(_this);
    return _this;
  }

  createClass(EffectDropdown, [{
    key: 'componentWillUpdate',
    value: function componentWillUpdate(nextProps, nextState) {
      var active = this.state.active;


      if (this.props.autoHide && active !== nextState.active) {
        if (!active) {
          document.body.addEventListener('click', this.handleBodyClick);
        } else {
          document.body.removeEventListener('click', this.handleBodyClick);
        }
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.props.autoHide) {
        document.body.removeEventListener('click', this.handleBodyClick);
      }
    }
  }, {
    key: 'toggle',
    value: function toggle() {
      this.setState({
        active: !this.state.active
      });
    }
  }, {
    key: 'hide',
    value: function hide() {
      this.setState({
        active: false
      });
    }
  }, {
    key: 'handleBodyClick',
    value: function handleBodyClick(event) {
      var notOutside = this.$el.contains(event.target);

      if (!notOutside) {
        this.setState({
          active: false
        });
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          effect = _props.effect,
          gutter = _props.gutter,
          autoHide = _props.autoHide,
          children = _props.children;
      var active = this.state.active;

      var dropdownItemProps = {
        effect: effect,
        gutter: gutter,
        autoHide: autoHide,
        active: active,
        itemCount: React__default.Children.count(children)
      };

      return React__default.createElement(
        'div',
        { ref: function ref(el) {
            _this2.$el = el;
          }, className: this.dropdownCls, role: 'menu' },
        React__default.createElement(
          'span',
          {
            role: 'menubar',
            tabIndex: 0,
            className: this.labelCls,
            style: this.labelStyle,
            onClick: this.toggle
          },
          this.props.label,
          React__default.createElement('i', { className: 'effect-dropdown__label-caret effect-dropdown-iconfont' })
        ),
        React__default.createElement(
          'ul',
          { className: 'effect-dropdown__content' },
          React__default.Children.map(children, function (c, index) {
            var newChildrenProps = _extends({}, c.props, dropdownItemProps, {
              key: c.key || index,
              index: index,
              handleHide: _this2.hide
            });

            return React__default.cloneElement(c, newChildrenProps);
          })
        )
      );
    }
  }, {
    key: 'dropdownCls',
    get: function get$$1() {
      return classnames({
        'effect-dropdown': true,
        'effect-dropdown--active': this.state.active
      });
    }
  }, {
    key: 'labelCls',
    get: function get$$1() {
      return classnames({
        'effect-dropdown__label': true,
        'effect-dropdown__label--raise': this.props.raiseLabel
      });
    }
  }, {
    key: 'labelStyle',
    get: function get$$1() {
      return {
        color: this.state.active ? this.props.activeColor : null
      };
    }
  }]);
  return EffectDropdown;
}(React.Component);

EffectDropdown.propTypes = {
  label: PropTypes.string,
  raiseLabel: PropTypes.bool,
  effect: PropTypes.oneOf(['simple', 'random', 'camber', 'stagger', 'fence']),
  gutter: PropTypes.number,
  activeColor: PropTypes.string,
  autoHide: PropTypes.bool,
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.element), PropTypes.object])
};

EffectDropdown.defaultProps = {
  effect: 'simple',
  label: '',
  raiseLabel: false,
  gutter: 5,
  activeColor: '#fc756f',
  autoHide: true,
  children: null
};

var BASE = {
  zIndex: 1500,
  offset: 3,
  itemHeight: 60
};

var EffectDropdownItem = function (_Component) {
  inherits(EffectDropdownItem, _Component);

  function EffectDropdownItem(props) {
    classCallCheck(this, EffectDropdownItem);

    var _this = possibleConstructorReturn(this, (EffectDropdownItem.__proto__ || Object.getPrototypeOf(EffectDropdownItem)).call(this, props));

    _this.state = {
      hover: false,
      randomLeft: '',
      randomTransform: _this.transform
    };

    _this.handleClick = _this.handleClick.bind(_this);
    _this.handleMouseEnter = _this.handleMouseEnter.bind(_this);
    _this.handleMouseLeave = _this.handleMouseLeave.bind(_this);
    return _this;
  }

  createClass(EffectDropdownItem, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _props = this.props,
          effect = _props.effect,
          active = _props.active;

      var nextActive = nextProps.active;

      if (active !== nextActive) {
        this.setState({
          hover: false
        });

        if (effect === 'random') {
          this.setState({
            randomLeft: nextActive ? Math.floor(Math.random() * 10 - 5) + 'px' : 0,
            randomTransform: this.getRandomTransform(nextActive)
          });
        }
      }
    }
  }, {
    key: 'getRandomTransform',
    value: function getRandomTransform() {
      var active = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      var scale = 1 - this.props.index / 100;

      var value = active ? {
        scale3d: [1, 1, 1],
        rotate: (Math.random() * 10 - 5).toFixed(2) + 'deg'
      } : {
        scale3d: [scale, scale, 1],
        rotate: 0
      };

      return transform(value);
    }
  }, {
    key: 'handleClick',
    value: function handleClick(event) {
      var _props2 = this.props,
          disabled = _props2.disabled,
          autoHide = _props2.autoHide,
          handleHide = _props2.handleHide,
          onClick = _props2.onClick;


      if (disabled) return;

      onClick && onClick({
        item: this,
        index: this.props.index,
        event: event
      });

      autoHide && handleHide();
    }
  }, {
    key: 'handleMouseEnter',
    value: function handleMouseEnter(event) {
      var _props3 = this.props,
          disabled = _props3.disabled,
          active = _props3.active,
          onMouseEnter = _props3.onMouseEnter;


      if (disabled) return;
      if (!active) return;

      this.setState({
        hover: true
      });

      onMouseEnter && onMouseEnter({
        item: this,
        index: this.props.index,
        event: event
      });
    }
  }, {
    key: 'handleMouseLeave',
    value: function handleMouseLeave(event) {
      var _props4 = this.props,
          disabled = _props4.disabled,
          onMouseLeave = _props4.onMouseLeave;


      if (disabled) return;

      this.setState({
        hover: false
      });

      onMouseLeave && onMouseLeave({
        item: this,
        index: this.props.index,
        event: event
      });
    }
  }, {
    key: 'render',
    value: function render() {
      return React__default.createElement(
        'li',
        {
          role: 'menuitem',
          style: this.itemStyle,
          onClick: this.handleClick,
          className: this.classname,
          onMouseEnter: this.handleMouseEnter,
          onMouseLeave: this.handleMouseLeave
        },
        React__default.createElement(
          'span',
          {
            style: this.labelStyle,
            className: 'effect-dropdown-item__label'
          },
          this.props.children
        )
      );
    }
  }, {
    key: 'transform',
    get: function get$$1() {
      var _props5 = this.props,
          effect = _props5.effect,
          index = _props5.index,
          active = _props5.active;

      var scale = 1 - index / 100;

      var commonOptions = {
        default: { scale3d: [scale, scale, 1] },
        active: { scale3d: [1, 1, 1] }
      };
      var optionsMap = {
        simple: _extends({}, commonOptions),
        random: {
          default: _extends({}, commonOptions.default, {
            rotate: 0
          }),
          active: _extends({}, commonOptions.active, {
            rotate: (Math.random() * 10 - 5).toFixed(2) + 'deg'
          })
        },
        camber: {
          default: _extends({}, commonOptions.default, {
            rotate: 0
          }),
          active: _extends({}, commonOptions.active, {
            rotate: index * 5 + 'deg'
          })
        },
        stagger: {
          default: {
            translate3d: [index % 2 ? '-40%' : '40%', 0, 0]
          },
          active: {
            translate3d: [0, 0, 0]
          }
        },
        fence: {
          default: {
            translate3d: [index % 2 ? '-50%' : '50%', 0, 0]
          },
          active: {
            translate3d: [0, 0, 0]
          }
        }
      };

      var defaultTransform = transform(optionsMap[effect].default);

      var activeTransfrom = transform(optionsMap[effect].active);

      return active ? activeTransfrom : defaultTransform;
    }
  }, {
    key: 'itemStyle',
    get: function get$$1() {
      var _props6 = this.props,
          effect = _props6.effect,
          index = _props6.index,
          active = _props6.active,
          itemCount = _props6.itemCount,
          gutter = _props6.gutter;


      var top = active ? (index + 1) * (BASE.itemHeight + gutter) + 'px' : BASE.offset * index + 'px';

      var commonStyle = {
        top: top,
        transform: this.props.effect === 'random' ? this.state.randomTransform : this.transform,
        zIndex: BASE.zIndex - (index + 1)
      };

      var styleMap = {
        simple: {},
        random: {
          left: this.state.randomLeft,
          transitionDelay: active ? (itemCount - index - 1) * 100 + 'ms' : index * 100 + 'ms'
        },
        camber: {
          top: active ? BASE.itemHeight + gutter + 'px' : commonStyle.top,
          transitionDelay: active ? (itemCount - index - 1) * 50 + 'ms' : index * 50 + 'ms'
        },
        stagger: {
          top: (index + 1) * (BASE.itemHeight + gutter) + 'px',
          opacity: Number(active)
        },
        fence: {
          top: (index + 1) * (BASE.itemHeight + gutter) + 'px',
          opacity: Number(active),
          transitionDelay: active ? index * 100 + 'ms' : (itemCount - index - 1) * 100 + 'ms'
        }
      };

      return _extends({}, commonStyle, styleMap[effect]);
    }
  }, {
    key: 'classname',
    get: function get$$1() {
      var _classnames;

      return classnames((_classnames = {
        'effect-dropdown-item': true
      }, defineProperty(_classnames, 'effect-dropdown-item--' + this.props.effect, true), defineProperty(_classnames, 'effect-dropdown-item--active', this.props.active), defineProperty(_classnames, 'effect-dropdown-item--disabled', this.props.disabled), _classnames));
    }
  }, {
    key: 'labelStyle',
    get: function get$$1() {
      return {
        color: this.state.hover ? '#fff' : null,
        backgroundColor: this.state.hover ? this.props.activeColor : null
      };
    }
  }]);
  return EffectDropdownItem;
}(React.Component);

EffectDropdownItem.propTypes = {
  disabled: PropTypes.bool,
  effect: PropTypes.oneOf(['simple', 'random', 'camber', 'stagger', 'fence']),
  gutter: PropTypes.number,
  index: PropTypes.number,
  activeColor: PropTypes.string,
  autoHide: PropTypes.bool,
  active: PropTypes.bool,
  itemCount: PropTypes.number,
  children: PropTypes.node,

  handleHide: PropTypes.func,
  onClick: PropTypes.func,
  onMouseEnter: PropTypes.func,
  onMouseLeave: PropTypes.func
};

EffectDropdownItem.defaultProps = {
  disabled: false,
  gutter: 5,
  activeColor: '#fc756f',
  autoHide: false,
  active: false,
  itemCount: 0,
  index: 0,
  effect: 'simple',
  children: null,

  handleHide: null,
  onClick: null,
  onMouseEnter: null,
  onMouseLeave: null
};

exports.EffectDropdown = EffectDropdown;
exports.EffectDropdownItem = EffectDropdownItem;
